from functools import wraps
from aioyoutube.exeptions import (
    VariableTypeError, VariableValueError
)


def search_validation(coroutine):
    @wraps(coroutine)
    async def wrapper(*args, **kwargs):
        """Decorator validate passed parameters for api method getting
        search result."""

        acceptable_order = (
            'date', 'rating', 'relevance', 'title', 'videoCount', 'viewCount',
        )
        acceptable_search_by = (
            'video', 'channel', 'playlist',
        )

        key = kwargs.get('key')
        text = kwargs.get('text')
        max_results = kwargs.get('max_results')
        page_token = kwargs.get('page_token')
        order = kwargs.get('order')
        published_after = kwargs.get('published_after')
        published_before = kwargs.get('published_before')
        search_by = kwargs.get('search_by')

        if key and not isinstance(key, str):
            raise VariableTypeError(
                f'Argument "key" must be an str, current type is {type(key)}.'
            )
        elif text and not isinstance(text, str):
            raise VariableTypeError(
                f'Argument "text" must be an str, current type is {type(text)}.'
            )
        elif max_results and not isinstance(max_results, int):
            raise VariableTypeError(
                'Argument "max_results" must be an int, current type is'
                f' {type(max_results)}.'
            )
        elif max_results and not 0 <= max_results <= 50:
            raise VariableValueError(
                'Argument "max_result" must be in range from 1 to 50, '
                f'current value is {max_results}.'
            )
        elif page_token and not isinstance(page_token, str):
            raise VariableTypeError(
                'Argument "page_token" must be an str, current type is '
                f'{type(page_token)}.'
            )
        elif order and not isinstance(order, str):
            raise VariableTypeError(
                'Argument "order" must be an str, current type is '
                f'{type(order)}.'
            )
        elif order and order not in acceptable_order:
            raise VariableValueError(
                'Acceptable values for argument "order" is '
                f'{acceptable_order}, current value is {order}.'
            )
        elif published_after and not isinstance(published_after, int):
            raise VariableTypeError(
                'Argument "published_after" must be an int, current type is '
                f'{type(published_after)}.'
            )
        elif published_before and not isinstance(published_before, int):
            raise VariableTypeError(
                'Argument "published_before" must be an int, current type is '
                f'{type(published_before)}.'
            )
        elif search_by and not isinstance(search_by, str):
            raise VariableTypeError(
                'Argument "search_by" must be an str, current type is '
                f'{type(search_by)}.'
            )
        elif search_by and search_by not in acceptable_search_by:
            raise VariableValueError(
                'Acceptable values for argument "search_by" is '
                f'{acceptable_search_by}, current value is {search_by}.'
            )

        return await coroutine(*args, **kwargs)

    return wrapper


def comment_threads_validation(coroutine):
    @wraps(coroutine)
    async def wrapper(*args, **kwargs):
        """Decorator validate passed parameters for api method getting
        video comment_threads."""
        acceptable_part = ('id', 'replies', 'snippet',)
        acceptable_order = ('time', 'relevance',)
        acceptable_text_format = ('plainText', 'html',)

        key = kwargs.get('key')
        part = kwargs.get('part')
        video_id = kwargs.get('video_id')
        max_results = kwargs.get('max_results')
        page_token = kwargs.get('page_token')
        order = kwargs.get('order')
        text_format = kwargs.get('text_format')
        search_text = kwargs.get('search_text')

        if key and not isinstance(key, str):
            raise VariableTypeError(
                f'Argument "key" must be an str, current type is {type(key)}.'
            )
        elif part and not isinstance(part, list):
            raise VariableTypeError(
                'Argument "part" must be an list, current type is'
                f' {type(part)}.'
            )
        elif part and not all(isinstance(item, str) for item in part):
            raise VariableTypeError(
                'Argument "part" must contain only str.'
            )
        elif part and not all(item in acceptable_part for item in part):
            raise VariableValueError(
                'Acceptable values for part contain parameter is '
                f'{acceptable_part}, current part contain {part}.'
            )
        elif max_results and not isinstance(max_results, int):
            raise VariableTypeError(
                'Argument "max_results" must be an int, current type is'
                f' {type(max_results)}.'
            )
        elif max_results and not 0 <= max_results <= 100:
            raise VariableValueError(
                'Argument "max_result" must be in range from 1 to 100, '
                f'current value is {max_results}.'
            )
        elif page_token and not isinstance(page_token, str):
            raise VariableTypeError(
                'Argument "page_token" must be an str, current type is '
                f'{type(page_token)}.'
            )
        elif order and not isinstance(order, str):
            raise VariableTypeError(
                'Argument "order" must be an str, current type is '
                f'{type(order)}.'
            )
        elif order and order not in acceptable_order:
            raise VariableValueError(
                'Acceptable values for argument "order" is '
                f'{acceptable_order}, current value is {order}.'
            )
        elif video_id and not isinstance(video_id, str):
            raise VariableTypeError(
                'Argument "video_id" must be an str, current type is'
                f' {type(video_id)}.'
            )
        elif text_format and not isinstance(text_format, str):
            raise VariableTypeError(
                'Argument "text_format" must be an str, current type is'
                f' {type(text_format)}.'
            )
        elif text_format and text_format not in acceptable_text_format:
            raise VariableValueError(
                'Acceptable values for argument "order" is '
                f'{acceptable_text_format}, current value is {text_format}.'
            )
        elif search_text and not isinstance(search_text, str):
            raise VariableTypeError(
                'Argument "search_text" must be an str, current type is'
                f' {type(search_text)}.'
            )

        return await coroutine(*args, **kwargs)

    return wrapper


def comments_validation(coroutine):
    @wraps(coroutine)
    async def wrapper(*args, **kwargs):
        """Decorator validate passed parameters for api method getting
        comments from comment_threads."""
        acceptable_part = ('id', 'snippet',)
        acceptable_text_format = ('plainText', 'html',)

        key = kwargs.get('key')
        part = kwargs.get('part')
        max_results = kwargs.get('max_results')
        page_token = kwargs.get('page_token')
        parent_id = kwargs.get('parent_id')
        text_format = kwargs.get('text_format')

        if key and not isinstance(key, str):
            raise VariableTypeError(
                f'Argument "key" must be an str, current type is {type(key)}.'
            )
        elif part and not isinstance(part, list):
            raise VariableTypeError(
                'Argument "part" must be an list, current type is'
                f' {type(part)}.'
            )
        elif part and not all(isinstance(item, str) for item in part):
            raise VariableTypeError(
                'Argument "part" must contain only str.'
            )
        elif part and not all(item in acceptable_part for item in part):
            raise VariableValueError(
                'Acceptable values for part contain parameter is '
                f'{acceptable_part}, current part contain {part}.'
            )
        elif max_results and not isinstance(max_results, int):
            raise VariableTypeError(
                'Argument "max_results" must be an int, current type is'
                f' {type(max_results)}.'
            )
        elif max_results and not 0 <= max_results <= 100:
            raise VariableValueError(
                'Argument "max_result" must be in range from 1 to 100, '
                f'current value is {max_results}.'
            )
        elif page_token and not isinstance(page_token, str):
            raise VariableTypeError(
                'Argument "page_token" must be an str, current type is '
                f'{type(page_token)}.'
            )
        elif text_format and not isinstance(text_format, str):
            raise VariableTypeError(
                'Argument "text_format" must be an str, current type is'
                f' {type(text_format)}.'
            )
        elif text_format and text_format not in acceptable_text_format:
            raise VariableValueError(
                'Acceptable values for argument "order" is '
                f'{acceptable_text_format}, current value is {text_format}.'
            )
        elif parent_id and not isinstance(parent_id, str):
            raise VariableTypeError(
                'Argument "parent_id" must be an str, current type is'
                f' {type(parent_id)}.'
            )

        return await coroutine(*args, **kwargs)

    return wrapper


def channels_validation(coroutine):
    @wraps(coroutine)
    async def wrapper(*args, **kwargs):
        """Decorator validate passed parameters for api method getting
            channels data."""
        acceptable_part = (
            'brandingSettings', 'contentDetails', 'contentOwnerDetails', 'id',
            'localizations', 'snippet', 'statistics', 'status', 'topicDetails',
        )

        key = kwargs.get('key')
        part = kwargs.get('part')
        max_results = kwargs.get('max_results')
        channel_id = kwargs.get('channel_id')
        user_name = kwargs.get('user_name')

        if key and not isinstance(key, str):
            raise VariableTypeError(
                f'Argument "key" must be an str, current type is {type(key)}.'
            )
        elif part and not isinstance(part, list):
            raise VariableTypeError(
                'Argument "part" must be an list, current type is'
                f' {type(part)}.'
            )
        elif part and not all(isinstance(item, str) for item in part):
            raise VariableTypeError(
                'Argument "part" must contain only str.'
            )
        elif part and not all(item in acceptable_part for item in part):
            raise VariableValueError(
                'Acceptable values for part contain parameter is '
                f'{acceptable_part}, current part contain {part}.'
            )
        elif max_results and not isinstance(max_results, int):
            raise VariableTypeError(
                'Argument "max_results" must be an int, current type is'
                f' {type(max_results)}.'
            )
        elif max_results and not 0 <= max_results <= 50:
            raise VariableValueError(
                'Argument "max_result" must be in range from 1 to 50, '
                f'current value is {max_results}.'
            )
        elif channel_id and user_name:
            raise VariableValueError(
                'Variable "channel_id" and "user_name" is not compatible, '
                'pass only one of them.'
            )
        elif channel_id and not isinstance(channel_id, str):
            raise VariableTypeError(
                'Argument "channel_id" must be an str, current type'
                f' is {type(channel_id)}.'
            )
        elif user_name and not isinstance(user_name, str):
            raise VariableTypeError(
                'Argument "user_name" must be an str, current type'
                f' is {type(user_name)}.'
            )

        return await coroutine(*args, **kwargs)

    return wrapper


def playlist_items_validation(coroutine):
    @wraps(coroutine)
    async def wrapper(*args, **kwargs):
        """Decorator validate passed parameters for api method getting
            playlist items data."""
        acceptable_part = (
            'contentDetails', 'id', 'snippet', 'status',
        )

        key = kwargs.get('key')
        part = kwargs.get('part')
        max_results = kwargs.get('max_results')
        playlist_id = kwargs.get('playlist_id')
        page_token = kwargs.get('page_token')

        if key and not isinstance(key, str):
            raise VariableTypeError(
                f'Argument "key" must be an str, current type is {type(key)}.'
            )
        elif part and not isinstance(part, list):
            raise VariableTypeError(
                'Argument "part" must be an list, current type is'
                f' {type(part)}.'
            )
        elif part and not all(isinstance(item, str) for item in part):
            raise VariableTypeError(
                'Argument "part" must contain only str.'
            )
        elif part and not all(item in acceptable_part for item in part):
            raise VariableValueError(
                'Acceptable values for part contain parameter is '
                f'{acceptable_part}, current part contain {part}.'
            )
        elif max_results and not isinstance(max_results, int):
            raise VariableTypeError(
                'Argument "max_results" must be an int, current type is'
                f' {type(max_results)}.'
            )
        elif max_results and not 0 <= max_results <= 50:
            raise VariableValueError(
                'Argument "max_result" must be in range from 1 to 50, '
                f'current value is {max_results}.'
            )
        elif playlist_id and not isinstance(playlist_id, str):
            raise VariableTypeError(
                'Argument "playlist_id" must be an str, current type is'
                f' {type(playlist_id)}.'
            )
        elif page_token and not isinstance(page_token, str):
            raise VariableTypeError(
                'Argument "page_token" must be an str, current type is '
                f'{type(page_token)}.'
            )

        return await coroutine(*args, **kwargs)

    return wrapper


def playlists_validation(coroutine):
    @wraps(coroutine)
    async def wrapper(*args, **kwargs):
        """Decorator validate passed parameters for api method getting
            playlists data."""
        acceptable_part = (
            'contentDetails', 'id', 'snippet', 'status', 'localizations',
            'player',
        )

        key = kwargs.get('key')
        part = kwargs.get('part')
        max_results = kwargs.get('max_results')
        channel_id = kwargs.get('channel_id')
        page_token = kwargs.get('page_token')

        if key and not isinstance(key, str):
            raise VariableTypeError(
                f'Argument "key" must be an str, current type is {type(key)}.'
            )
        elif part and not isinstance(part, list):
            raise VariableTypeError(
                'Argument "part" must be an list, current type is'
                f' {type(part)}.'
            )
        elif part and not all(isinstance(item, str) for item in part):
            raise VariableTypeError(
                'Argument "part" must contain only str.'
            )
        elif part and not all(item in acceptable_part for item in part):
            raise VariableValueError(
                'Acceptable values for part contain parameter is '
                f'{acceptable_part}, current part contain {part}.'
            )
        elif max_results and not isinstance(max_results, int):
            raise VariableTypeError(
                'Argument "max_results" must be an int, current type is'
                f' {type(max_results)}.'
            )
        elif max_results and not 0 <= max_results <= 50:
            raise VariableValueError(
                'Argument "max_result" must be in range from 1 to 50, '
                f'current value is {max_results}.'
            )
        elif channel_id and not isinstance(channel_id, str):
            raise VariableTypeError(
                'Argument "channel_id" must be an str, current type is'
                f' {type(channel_id)}.'
            )
        elif page_token and not isinstance(page_token, str):
            raise VariableTypeError(
                'Argument "page_token" must be an str, current type is '
                f'{type(page_token)}.'
            )

        return await coroutine(*args, **kwargs)

    return wrapper


def videos_validation(coroutine):
    @wraps(coroutine)
    async def wrapper(*args, **kwargs):
        """Decorator validate passed parameters for api method getting
            video data."""
        acceptable_part = (
            'contentDetails', 'id', 'liveStreamingDetails', 'localizations',
            'player', 'recordingDetails', 'snippet', 'statistics',
            'status', 'topicDetails',
        )

        key = kwargs.get('key')
        part = kwargs.get('part')
        max_results = kwargs.get('max_results')
        video_ids = kwargs.get('video_ids')
        page_token = kwargs.get('page_token')

        if key and not isinstance(key, str):
            raise VariableTypeError(
                f'Argument "key" must be an str, current type is {type(key)}.'
            )
        elif part and not isinstance(part, list):
            raise VariableTypeError(
                'Argument "part" must be an list, current type is'
                f' {type(part)}.'
            )
        elif part and not all(isinstance(item, str) for item in part):
            raise VariableTypeError(
                'Argument "part" must contain only str.'
            )
        elif part and not all(item in acceptable_part for item in part):
            raise VariableValueError(
                'Acceptable values for part contain parameter is '
                f'{acceptable_part}, current part contain {part}.'
            )
        elif max_results and not isinstance(max_results, int):
            raise VariableTypeError(
                'Argument "max_results" must be an int, current type is'
                f' {type(max_results)}.'
            )
        elif max_results and not 0 <= max_results <= 50:
            raise VariableValueError(
                'Argument "max_result" must be in range from 1 to 50, '
                f'current value is {max_results}.'
            )
        elif video_ids and not isinstance(video_ids, list):
            raise VariableTypeError(
                'Argument "video_ids" must be an list, current type is'
                f' {type(video_ids)}.'
            )
        elif video_ids and not all(isinstance(item, str) for item in video_ids):
            raise VariableTypeError(
                'Argument "video_ids" must contain only str.'
            )
        elif page_token and not isinstance(page_token, str):
            raise VariableTypeError(
                'Argument "page_token" must be an str, current type is '
                f'{type(page_token)}.'
            )

        return await coroutine(*args, **kwargs)

    return wrapper



from functools import wraps
from rfc3339 import rfc3339
from datetime import datetime


def insert_name(coroutine):
    @wraps(coroutine)
    async def wrapper(*args, **kwargs):
        """Decorator insert class method name into this method parameters."""
        return await coroutine(*args, **kwargs, name=coroutine.__name__)

    return wrapper


def time_converting(time: int):
    """Converting time format from unixtime to rfc3339."""
    return rfc3339(datetime.fromtimestamp(time), utc=True)



from aiohttp.client_exceptions import ContentTypeError
from functools import wraps

from aioyoutube.exeptions import *


def response_error_handler(coroutine):
    @wraps(coroutine)
    async def wrapper(*args, **kwargs):
        """Decorator checker api response."""

        try:
            json = await coroutine(*args, **kwargs)
        except ContentTypeError as err:
            url = err.request_info.url
            raise WrongApiName(mess=(
                f'Wrong api name "{url.parts[-1]}" has been passed '
                f'into request url.'
            ))

        try:
            reason = json['error']['errors'][0]['reason']
            message = json['error']['message']
            code = json['error']['code']
        except KeyError:
            return json
        else:
            if code == 400:
                if reason == 'keyInvalid':
                    key = kwargs.get('key')
                    raise InvalidApiKey(code=400, json=json, mess=(
                        f'Passed youtube api key:{key} is invalid.'
                    ))
                elif reason == 'unknownPart':
                    raise UnknownField(code=400, json=json, mess=(
                        f'Unknown parameter: {message} has been passed.'
                    ))
                elif reason == 'invalidParameter':
                    raise InvalidParameterValue(code=400, json=json, mess=(
                        'One of passed parameters value is not '
                        f'allowed: {message}.'
                    ))
                elif reason == 'invalidPageToken':
                    token = kwargs.get('page_token')
                    raise InvalidPageToken(code=400, json=json, mess=(
                        f'Passed invalid page token {token}.'
                    ))
                elif reason == 'processingFailure':
                    raise ProcessingFailure(code=400, json=json, mess=(
                        "Youtube can't process request, try repeat after "
                        "some time later."
                    ))
                elif reason == 'missingRequiredParameter':
                    raise MissingRequiredParameter(code=400, json=json, mess=(
                        "Some required parameter has been miss."
                    ))
                elif reason == 'unexpectedParameter':
                    raise UnexpectedParameter(code=400, json=json, mess=(
                        'Unexpected parameter has been passed.'
                    ))

            elif code == 401:
                if reason == 'authorizationRequired':
                    raise NoAuthorized(code=401, json=json, mess=(
                        'Request required OAuth2 authorization.'
                    ))

            elif code == 403:
                if reason in ('dailyLimitExceededUnreg', 'quotaExceeded',
                              'dailyLimitExceeded',):
                    raise ExceededDailyLimit(code=403, json=json, mess=(
                        f'Day request limit for api key {kwargs.get("key")} '
                        'was exceeded.'
                    ))
                elif reason == 'channelClosed':
                    raise ChannelClosed(code=403, json=json, mess=(
                        f'Channel {kwargs.get("channel_id")} was closed.'
                    ))
                elif reason == 'channelSuspended':
                    raise ChannelSuspended(code=403, json=json, mess=(
                        f'Channel {kwargs.get("channel_id")} was suspended.'
                    ))
                elif reason == 'forbidden':
                    raise ForbiddenError(code=403, json=json, mess=(
                        f'Some of the parameter passed: {args}, {kwargs}, work'
                        ' only with OAuth2 authenticated app.'
                    ))

            elif code == 404:
                if reason == 'channelNotFound':
                    raise ChannelNotExist(code=404, json=json, mess=(
                        f"Channel {kwargs.get('channel_id')} doesn't exist."
                    ))
                elif reason == 'contentOwnerAccountNotFound':
                    raise InvalidOwnerAccount(code=404, json=json, mess=(
                        f"Owner account not found."
                    ))
                elif reason == 'videoNotFound':
                    raise InvalidVideoId(code=404, json=json, mess=(
                        f"Video with passed id:{kwargs.get('id')} "
                        "doesn't exist."
                    ))

    return wrapper


def comment_threads_error_handler(coroutine):
    @wraps(coroutine)
    async def wrapper(*args, **kwargs):
        """Decorator checker api comments response."""
        video_id = kwargs.get('video_id')

        json = await coroutine(*args, **kwargs)

        if json.get('error'):
            reason = json['error']['errors'][0]['reason']

            if json['error']['code'] == 403:
                if reason == 'commentsDisabled':
                    raise CommentsDisabled(code=403, json=json, mess=(
                        f'Comments for video: {video_id}, is disabled.'
                    ))

        return json

    return wrapper


def comments_error_handler(coroutine):
    @wraps(coroutine)
    async def wrapper(*args, **kwargs):
        """Decorator checker api comments response."""
        parent_id = kwargs.get('parentId')

        json = await coroutine(*args, **kwargs)

        if json.get('error'):
            reason = json['error']['errors'][0]['reason']

            if json['error']['code'] == 404:
                if reason == 'commentNotFound':
                    raise InvalidCommentThreadId(code=404, json=json, mess=(
                        f"Comment thread with id={parent_id} doesn't exist."
                    ))

        return json

    return wrapper


def channels_error_handler(coroutine):
    @wraps(coroutine)
    async def wrapper(*args, **kwargs):
        """Decorator checker api channels response."""
        channel_id = kwargs.get('channel_id')
        user_name = kwargs.get('user_name')

        json = await coroutine(*args, **kwargs)

        if not json.get('items') and not json.get('error'):
            if user_name:
                raise InvalidUserName(json=json, mess=(
                    f"Channel with user name {user_name} doesn't exist."
                ))
            elif channel_id:
                raise ChannelNotExist(code=404, json=json, mess=(
                    f"Channel with id {channel_id} doesn't exist."
                ))

        elif json.get('error'):
            reason = json['error']['errors'][0]['reason']

            if json['error']['code'] == 400:
                if reason == 'invalidChannelId':
                    raise InvalidChannelId(code=400, json=json, mess=(
                        f'Passed channel id parameter: {channel_id} is invalid.'
                    ))

            if json['error']['code'] == 403:
                if reason == 'channelForbidden':
                    raise ChannelForbidden(code=403, json=json, mess=(
                        f"Channel with id {channel_id} doesn't support "
                        "requests."
                    ))

        return json

    return wrapper


def playlist_items_error_handler(coroutine):
    @wraps(coroutine)
    async def wrapper(*args, **kwargs):
        """Decorator checker api playlist items response."""
        playlist_id = kwargs.get('playlist_id')

        json = await coroutine(*args, **kwargs)

        if json.get('error'):
            reason = json['error']['errors'][0]['reason']

            if json['error']['code'] == 403:
                if reason == 'playlistItemsNotAccessible':
                    raise PlaylistItemsNotAccessible(code=403, json=json, mess=(
                        f'Playlist with id={playlist_id} required OAuth2 '
                        f'authorization.'
                    ))
                elif reason == 'watchHistoryNotAccessible':
                    raise HistoryNotAccessible(code=403, json=json, mess=(
                        f"Items of playlist 'history' can't be get through API."
                    ))
                elif reason == 'watchLaterNotAccessible':
                    raise WatchLaterNotAccessible(code=403, json=json, mess=(
                        f"Items of playlist 'watch later' can't be get "
                        f"through API."
                    ))

            if json['error']['code'] == 404:
                if reason == 'playlistNotFound':
                    raise InvalidPlaylistId(code=404, json=json, mess=(
                        f"Playlist with id {kwargs.get('playlist_id')} "
                        "doesn't exist."
                    ))

        return json

    return wrapper


def playlist_error_handler(coroutine):
    @wraps(coroutine)
    async def wrapper(*args, **kwargs):
        """Decorator checker api playlist response."""

        json = await coroutine(*args, **kwargs)

        if json.get('error'):
            reason = json['error']['errors'][0]['reason']

            if json['error']['code'] == 403:
                if reason == 'playlistForbidden':
                    raise PlaylistForbidden(code=403, json=json, mess=(
                        "Request playlist required OAuth2 authentication "
                        "or doesn't support api requests."
                    ))

        return json

    return wrapper



from setuptools import setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='aioyoutube',
    version='0.1.5',
    packages=['aioyoutube', 'aioyoutube.helpers', 'aioyoutube.handlers',
              'aioyoutube.exeptions'],
    url='https://github.com/diarts/aioyoutube.git',
    license='MIT',
    author='konstantin',
    author_email='korobanov1993@yandex.ru',
    description='Async wrapper for REST API youtube.com',
    long_description=long_description,
    long_description_content_type="text/markdown",
    classifiers=[
        "Programming Language :: Python :: 3",
    ],
    python_requires='>=3.7.2',
)
